#!/bin/bash

verbose="0"
my_server="hemera"
my_owner="/bigdata/ParticleFlows/Gregory"
my_contact_angle="90"
my_side="-1"
my_ec="-1"
copy_data="0"
export_data="0"
every="1"
make_figure="0"
few_only="0" #useful when doing home office

#export PATH=$PATH:/home/gl5972/Leipzig/FigX/src
#bash ExportData.sh --server -1 --owner . --ca 90 --side 3 --ec 2 --export

function PrintUsage()
{
	echo "Copy data from server and make Figure X"
	echo "--help:    help"
	echo "--verbose: verbose"
	echo "--server:  ssh server"
	echo "--owner:   directory on the server"
	echo "--ca:      contact angle"
	echo "--side:    copy/export data from side=X only"
	echo "--ec:      copy/export data from ec=X only"
	echo "--copy:    copy data"
	echo "--export:  export to paraview"
	echo "--every:   export every X"
	echo "--fig:     make figure"
	exit
}

function PassArgs()
{
	local shorts='hvs:o:c:d:e:pxy:f'
	local longs='help,verbose,server:,owner:,ca:,side:,ec:,copy,export,every:,fig'
	local options=$(getopt --options ${shorts} --long ${longs} --name "$0" -- "$@")
	if [ $? != 0 ]; then echo "Failed parsing options." >&2 ; exit 1 ; fi
	eval set -- "${options}"
	while true
	do
		case "$1" in
			-h | --help)    PrintUsage;          shift  ;;
			-v | --verbose) verbose="1";         shift  ;;
			-s | --server)  my_server=$2;        shift 2;;
			-o | --owner)   my_owner=$2;         shift 2;;
			-c | --ca)      my_contact_angle=$2; shift 2;;
			-d | --side)    my_side=$2;          shift 2;;
			-e | --ec)      my_ec=$2;            shift 2;;
			-p | --copy)    copy_data="1";       shift  ;;
			-x | --export)  export_data="1";     shift  ;;
			-y | --every)   every=$2;            shift 2;;
			-f | --fig)     make_figure="1";     shift  ;;
			--) shift ; break ;;
			* ) break ;;
		esac
	done
}

function Main()
{
	PassArgs "$0" "$@"
	if (( $(echo "${my_server} < 0" | bc -l) ))
	then
		module load gcc/7.3.0 openmpi/2.1.2 make/4.3 cmake/3.20.2 python/3.8.0 blas/3.8.0 lapack/3.8.0 boost/1.78.0 git/2.30.1
		IGL_NUM_THREADS=1
		LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:${HOME}/local/libconfig-1.7.3/lib
		LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:${HOME}/local/gmp-6.2.1/lib
		LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:${HOME}/local/mpfr-4.2.0/lib
		PATH=$PATH:${HOME}/Leipzig/FigX/src
	fi
	echo "contact angle = ${my_contact_angle} deg"
	local side_folders="${my_owner}/ca${my_contact_angle}/side${my_side}"
	(( ${my_side} < 0 )) && side_folders=$(GetSideFolders "${my_server}" "${my_owner}/ca${my_contact_angle}") #side_folders = path + / + [side3,side4,...]
	for side_folder in ${side_folders}
	do
		local side=$(echo ${side_folder} | awk -F '/' '{print $NF}' | sed s'/side//')
		local ecs=${my_ec}
		(( $(echo "${my_ec} < 0" | bc -l) )) && ecs=$(GetEcs ${my_server} ${side_folder} ${few_only})
		for ec in ${ecs}
		do
			if (( ${copy_data} ))
			then
				CopyFields ${my_server} ${my_owner} ${my_contact_angle} ${side} ${ec} ${few_only}
			fi
 			if (( ${export_data} ))
 			then
				ExportFieldsToParaview ${my_contact_angle} ${side} ${ec} ${few_only}
 			fi
		done
	done
# 	if (( ${make_figure} )); then python3 figX.py; fi
}

function GetSideFolders()
{
	local arg_server=${1}
	local arg_owner=${2}
	if (( $(echo "${arg_server} < 0" | bc -l) ))
	then
		#Get directories sideX on local machine, X = 3,4,5,...
		local empty=$(ls -A ${arg_owner})
		[ ! -z "${empty}" ] && ls -d ${arg_owner}/side[0-9]*
	else
		#Get directories sideX on ssh server, X = 3,4,5,...
		local empty=$(ssh ${arg_server} "ls -A ${arg_owner}")
		[ ! -z "${empty}" ] && ssh ${arg_server} "ls -d ${arg_owner}/side[0-9]*"
	fi
}

function GetEcs()
{
	local arg_server=${1}
	local arg_folder=${2}
	local arg_few=${3}
	if (( $(echo "${arg_server} < 0" | bc -l) ))
	then
		#Get directories ecX.Y on local machine
		local empty=$(ls -A ${arg_folder})
			if [ ! -z "${empty}" ]
			then
				local ecs=($(ls -d ${arg_folder}/ec[0-9]*)) #bash array
				(( ${arg_few}>0 )) && ecs=($(echo ${ecs[@]} | awk -F' ' '{ for (i=2;i<=NF;i+=2) print $i}')) #bash array
				for i in "${!ecs[@]}"
				do
					local ec=$(echo ${ecs[${i}]} | awk -F '/' '{print $NF}' | sed "s/^ec//")
					ecs[${i}]=${ec}
				done
				echo ${ecs[@]}
			fi
	else
		#Get directories ecX.Y on ssh server
		local empty=$(ssh ${arg_server} "ls -A ${arg_folder}")
		if [ ! -z "${empty}" ]
		then
			local ecs=($(ssh ${arg_server} "ls -d ${arg_folder}/ec[0-9]*")) #bash array
			(( ${arg_few}>0 )) && ecs=($(echo ${ecs[@]} | awk -F' ' '{ for (i=2;i<=NF;i+=2) print $i}')) #bash array
			for i in "${!ecs[@]}"
			do
				local ec=$(echo ${ecs[${i}]} | awk -F '/' '{print $NF}' | sed "s/^ec//")
				ecs[${i}]=${ec}
			done
			echo ${ecs[@]}
		fi
	fi
}

function CopyFields()
{
	local arg_server=${1}
	local arg_owner=${2}
	local arg_ca=${3}
	local arg_side=${4}
	local arg_ec=$(LC_NUMERIC="en_US.UTF-8"; printf "%.1f" ${5})
	local arg_few=${6}
	if (( $(echo "${arg_server} < 0" | bc -l) ))
	then
		echo "  Provide an ssh server"
	else
		local base_dir=$(dirname "$(realpath $0)")
		local local_dir="${base_dir}/ca${arg_ca}/side${arg_side}/ec${arg_ec}"
		local server_dir="${arg_owner}/ca${arg_ca}/side${arg_side}/ec${arg_ec}"
		echo "  copy side = ${arg_side} ec = ${arg_ec}"
		mkdir -p ${local_dir}
		scp ${arg_server}:${server_dir}/Input.in ${local_dir} &> /dev/null
		scp ${arg_server}:${server_dir}/force.txt ${local_dir} &> /dev/null
		scp ${arg_server}:${server_dir}/base.dat ${local_dir} &> /dev/null
		scp ${arg_server}:${server_dir}/base.stl ${local_dir} &> /dev/null #Test
		scp ${arg_server}:${server_dir}/base.txt ${local_dir} &> /dev/null #Test
		scp ${arg_server}:${server_dir}/log_output ${local_dir} &> /dev/null
		if (( ${arg_few}>0 ))
		then
			echo "???"
# 			scp ${arg_server}:${server_dir}/phi_t???[02468]0.dat ${local_dir} &> /dev/null # Heavy stuff
# 			scp ${arg_server}:${server_dir}/drop_t???[02468]0.vtk ${local_dir} &> /dev/null
# 			scp ${arg_server}:${server_dir}/hinge0_t???[02468]0.dat ${local_dir} &> /dev/null
# 			scp ${arg_server}:${server_dir}/side0_t???[02468]0.dat ${local_dir} &> /dev/null
# 			scp ${arg_server}:${server_dir}/tpcl_t???[02468]0.dat ${local_dir} &> /dev/null
		else
# 			scp ${arg_server}:${server_dir}/phi_t?????.dat ${local_dir} &> /dev/null # Heavy stuff
# 			scp ${arg_server}:${server_dir}/drop_t?????.vtk ${local_dir} &> /dev/null
# 			scp ${arg_server}:${server_dir}/hinge?_t?????.dat ${local_dir} &> /dev/null
# 			scp ${arg_server}:${server_dir}/side?_t?????.dat ${local_dir} &> /dev/null
# 			scp ${arg_server}:${server_dir}/tpcl_t?????.dat ${local_dir} &> /dev/null

			#Test
# 			scp ${arg_server}:${server_dir}/hinge?_t?????.txt ${local_dir} &> /dev/null
# 			scp ${arg_server}:${server_dir}/hinge?_t?????.stl ${local_dir} &> /dev/null
# 			scp ${arg_server}:${server_dir}/side?_t?????.txt ${local_dir} &> /dev/null
			scp ${arg_server}:${server_dir}/side?_t?????.stl ${local_dir} &> /dev/null
# 			scp ${arg_server}:${server_dir}/tpcl??_t?????.txt ${local_dir} &> /dev/null
# 			scp ${arg_server}:${server_dir}/drop_t?????.vtk ${local_dir} &> /dev/null
		fi
	fi
}

function ExportFieldsToParaview()
{
	local arg_ca=${1}
	local arg_side=${2}
	local arg_ec=$(LC_NUMERIC="en_US.UTF-8"; printf "%.1f" ${3})
	local arg_few=${4}
	(( ${arg_few}>0 )) && every="20"
	local base_dir=$(dirname "$(realpath $0)")
	local local_dir="${base_dir}/ca${arg_ca}/side${arg_side}/ec${arg_ec}"
	echo "  export side = ${arg_side} ec = ${arg_ec}"
	if [ -d "${local_dir}" ]
	then
		empty=$(find ${local_dir} -maxdepth 1 -name 'side0_t?????.dat' -printf '1' -quit)
		if [ -z "${empty}" ]
		then
			echo "  empty directory"
		else
			local latest_time=$(expr 0 + $(ls ${local_dir}/side0_t?????.dat | tail -n 1 | awk -F '/' '{print $NF}' | awk -vFS='(_t|.dat)' '{print $2}'))
			cd ${local_dir}
			if [ -f "base.dat" ]
			then
				ExportSolid -f base -n ${arg_side} -stl -txt &> /dev/null
			else
				echo "  base.dat does not exist"
			fi
			local smax=${arg_side}
			(( ${arg_few}>0 )) && smax="1" #side0 only
			for (( s=0; s<${smax}; s++ ))
			do
				ExportSolid -f side${s} -t 0:${latest_time}:${every} -tp -stl -txt &> /dev/null
				ExportHinge -f hinge${s} -t 0:${latest_time}:${every} &> /dev/null
			done
# 			ExportFieldToVTS -f phi -t 0:${latest_time}:${every} &> /dev/null # Heavy stuff
			ExportPolylines -f tpcl -t 0:${latest_time}:${every} &> /dev/null
			n_max=$(ls -1 tpcl??_t?????.txt | awk -F '/' '{print $NF}' | awk -vFS='(tpcl|_t)' '{print $2}' | tail -n 1)
			n_max=$(expr ${n_max} + 1)
			echo "    cloud size = ${n_max}"
			PolylineToVTK.py -f tpcl -n ${n_max} -t 0:${latest_time}:${every} &> /dev/null
			#for f in tpcl??_t?????.py; do pvpython $f; done
			cd ${base_dir}
		fi
	else
		echo "  directory does not exist"
	fi
}

Main "$@"
