#/bin/bash

my_src="$HOME/Leipzig/FigX/src"
my_ca="90"
my_side="-1"
my_queue="defq"

function PrintUsage()
{
	echo "Create drop in stl format"
	echo "--help:  help"
	echo "--ca:    contact angle"
	echo "--side:  side"
	echo "--src:   src directory"
	echo "--queue: hemera queue"
	exit
}

function PassArgs()
{
	local shorts='hc:s:r:q:'
	local longs='help,ca:,side:,src:,queue:'
	local options=$(getopt --options ${shorts} --long ${longs} --name "$0" -- "$@")
	if [ $? != 0 ]; then echo "Failed parsing options." >&2 ; exit 1 ; fi
	eval set -- "${options}"
	while true
	do
		case "$1" in
			-h | --help)  PrintUsage;  shift  ;;
			-c | --ca)    my_ca=$2;    shift 2;;
			-s | --side)  my_side=$2;  shift 2;;
			-r | --src)   my_src=$2;   shift 2;;
			-q | --queue) my_queue=$2; shift 2;;
			--) shift ; break ;;
			* ) break ;;
		esac
	done
}

function GetSides()
{
	local arg_path=${1}
	local arg_ca=${2}
	if [ -d ${arg_path}/ca${arg_ca} ]
	then
		local sides=($(ls -d ${arg_path}/ca${arg_ca}/side[0-9]*))
		for i in "${!sides[@]}" #bash array
		do
			local side=$(echo ${sides[${i}]} | awk -F '/' '{print $NF}' | sed "s/^side//")
			sides[${i}]=${side}
		done
		echo ${sides[@]}
	fi
}

function GetEcs()
{
	local arg_path=${1}
	local arg_ca=${2}
	local arg_side=${3}
	local empty=$(ls -A ${arg_path}/ca${arg_ca}/side${arg_side})
	if [ ! -z "${empty}" ]
	then
		local ecs=($(ls -d ${arg_path}/ca${arg_ca}/side${arg_side}/ec[0-9]*))
		for i in "${!ecs[@]}" #bash array
		do
			local ec=$(echo ${ecs[${i}]} | awk -F '/' '{print $NF}' | sed "s/^ec//")
			ecs[${i}]=${ec}
		done
		echo ${ecs[@]}
	fi
}

function GetLatestTime()
{
	local arg_folder=${1}
	test -e ${arg_folder}/side0_t00000.dat
	if [ $? -eq 0 ]
	then
		local latest_time=$(ls ${arg_folder}/side0_t?????.dat | tail -n 1 | awk -F '/' '{print $NF}' | awk -vFS='(_t|.dat)' '{print $2}')
		echo $(expr ${latest_time} + 1)
	else
		echo "-1"
	fi
}

function MakeHemeraFile()
{
	arg_path=${1}
	arg_ca=${2}
	arg_side=${3}
	arg_ec=${4}
	local local_path="${arg_path}/ca${arg_ca}/side${arg_side}/ec${arg_ec}"
	local local_file="${local_path}/drop.sh"
	local latest_time=$(GetLatestTime ${local_path})
	if [ -d ${local_path} ]
	then
		echo '#!/bin/bash' > ${local_file}
		echo '#SBATCH --job-name=drop' >> ${local_file}
		echo "#SBATCH --partition=${my_queue}" >> ${local_file}
		echo '#SBATCH --time=1:00:00' >> ${local_file}
		echo '#SBATCH --nodes=1' >> ${local_file}
		echo '#SBATCH --ntasks=1' >> ${local_file}
		echo '#SBATCH --error=drop_error' >> ${local_file}
		echo '#SBATCH --output=drop_output' >> ${local_file}
		echo 'module purge' >> ${local_file}
		echo 'module load gcc/7.3.0 openmpi/2.1.2 lapack/3.8.0 boost/1.78.0 paraview' >> ${local_file}
		echo 'export IGL_NUM_THREADS=1' >> ${local_file}
		echo 'export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$HOME/local/libconfig-1.7.3/lib' >> ${local_file}
		echo 'export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$HOME/local/gmp-6.2.1/lib' >> ${local_file}
		echo 'export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$HOME/local/mpfr-4.2.0/lib' >> ${local_file}
		echo 'cd $SLURM_SUBMIT_DIR' >> ${local_file}
		echo "mpirun -np 1 ${my_src}/ExportFieldToVTS -f phi -t 0:${latest_time}" >> ${local_file}
		echo "pvpython ${arg_path}/ExportDrop.py" >> ${local_file}
		echo 'rm -v phi_t?????.vts'  >> ${local_file}
	else
		echo "${local_path} does not exist"
	fi
}

function Main()
{
	PassArgs "$0" "$@"
	echo "contact angle = ${my_ca} deg"
	local base_path=$(dirname "$(realpath $0)")
	local sides=${my_side}
	(( ${my_side}<0 )) && sides=$(GetSides "${base_path}" "${my_ca}")
	if [ -z "${sides}" ]
	then
		echo "  ${base_path}/ca${my_ca} does not exist"
	else
		for side in ${sides}
		do
			ecs=$(GetEcs "${base_path}" "${my_ca}" "${side}")
			if [ -z "${ecs}" ]
			then
				echo "  ${base_path}/ca${my_ca}/side${side} does not own any ec folders"
			else
				for ec in ${ecs}
				do
					echo "  ca = ${my_ca} side = ${side} ec = ${ec}"
					MakeHemeraFile "${base_path}" "${my_ca}" "${side}" ${ec}
					cd ${base_path}/ca${my_ca}/side${side}/ec${ec}; sbatch drop.sh; cd ${base_path}
				done
			fi
		done
	fi
}

Main "$@"
