#!/usr/bin/python3
import numpy as np
from pyquaternion import Quaternion
import argparse
import matplotlib.pyplot as plt
import sys
from Utils import *

parser = argparse.ArgumentParser(description='Export hinge to vtk.')
parser.add_argument('-t',type=StrToNumpy,default='-1',help='time')
parser.add_argument('-f',type=str,default='hinge',help='file name')
parser.add_argument('-n',type=int,default='-1',help='hinge number')
parser.add_argument('-res',type=int,default=10,help='resolution')
parser.add_argument('-a',type=float,default=-1,help='apothem')
args = parser.parse_args()

def GetBeads(fname,time,a=-1):
	my_file=''
	if fname[-4:] == ".txt":
		my_file = fname
	else:
		tExtension = ''
		if time>-1:
			tExtension = '_t{t:05d}'.format(t=time)
		my_file = fname + tExtension + '.txt'
	rad = np.loadtxt(my_file ,usecols=(1,),unpack=True)
	pos = np.loadtxt(my_file,usecols=(4,5,6),unpack=True).transpose()
	if a>0:
		u = np.array([0,0,1])
		v = pos[-1] - pos[0]
		v = v/np.linalg.norm(v)
		w = np.cross(u,v)
		posa = np.array([posi + a*w/2 for posi in pos])
		posb = np.array([posi - a*w/2 for posi in pos])
		pos = np.concatenate((posa,posb))
		rad = np.concatenate((rad,rad))
	return pos,rad

def VTKScript(positions,radii,res,fname,time):
	original_stdout = sys.stdout
	tExtension = ''
	out = ''
	if fname[-4:] == ".txt":
		out = fname[:-4]
	else:
		if time>-1:
			tExtension = '_t{t:05d}'.format(t=time)
		out = fname + tExtension

	with open(out + '.py', 'w') as f:
		sys.stdout = f
		print("from paraview.simple import *")
		for b,(positionb,radb) in enumerate(zip(positions,radii)):
			vtkFileName = fname + '_b{b:02}_{time}.vtk'.format(b=b,time=tExtension)
			print("sphere{b} = Sphere()".format(b=b))
			print("sphere{b}.Center = [{xc}, {yc}, {zc}]".format(b=b,xc=positionb[0],yc=positionb[1],zc=positionb[2]))
			print("sphere{b}.Radius = {r}".format(b=b,r=radb))
			print("sphere{b}.PhiResolution = {res}".format(b=b,res=res))
			print("sphere{b}.ThetaResolution = {res}".format(b=b,res=res))
			print("SaveData('{out}', proxy=sphere{b})".format(b=b,out=vtkFileName))
		sys.stdout = original_stdout
	print("  pvpython",out + '.py')
	return(0)

def MakeVTKScripts(hname,a=-1):
	for t in args.t:
		positions,radii = GetBeads(hname,t,a)
		VTKScript(positions,radii,args.res,hname,t)
	if args.t.size>2:
		print('  for (( t={ta}; t<{tb}; t=t+{dt} )); do echo $t; pvpython {f}_t$(printf "%05d" $t).py; done'.format(ta=args.t[0],tb=args.t[-1]+1,dt=args.t[1]-args.t[0],f=hname))

if args.n<0:
	MakeVTKScripts(args.f,args.a)
else:
	for h in range(args.n):
		MakeVTKScripts(args.f+str(h),args.a)
