#!/bin/python3
import numpy as np
from pyquaternion import Quaternion
import argparse
import matplotlib.pyplot as plt

def StrToNumpy(s):
	if not s:
		return np.array([])
	else:
		l = list(map(int,s.split(':')))
		return np.array(l) if len(l)==1 else np.arange(l[0],l[1]) if len(l)==2 else np.arange(l[0],l[1],l[2])

parser = argparse.ArgumentParser(description='??')
parser.add_argument('-t',type=StrToNumpy,default='0:100',help='time')
parser.add_argument('-f',type=str,default='hinge',help='file name')
args = parser.parse_args()

def GetRadius(time):
	return np.loadtxt(args.f + '_t{time:05}.txt'.format(time=time),usecols=(1,),unpack=True)

def GetPosition(time):
	return np.loadtxt(args.f + '_t{time:05}.txt'.format(time=time),usecols=(4,5,6),unpack=True)

def GetQuaternion(time):
	#qi = Quaternion(w, x, y, z)
	qs = np.loadtxt(args.f + '_t{0:05}.txt'.format(time), usecols=(10,7,8,9), unpack=True).transpose()
	qs = np.array([Quaternion(qi) for qi in qs],dtype=Quaternion)
	return qs

#plot
fig, ax = plt.subplots()
ax.set_aspect('equal')

#first bead position
x0,_,y0 = GetPosition(args.t[0])
x0 = x0[0]
y0 = y0[0]

#Fiber length
L,_,_= GetPosition(args.t[0])
L = L[-1] - x0

#Plot bead COM at every time step
for i,t in enumerate(args.t):
	xs,_,ys = GetPosition(t)
	xs = (xs-x0)/L
	ys = (ys-y0)/L
	ax.plot(xs,ys,'.',markersize=1,color='black')

#Plot fiber at t = last time
lastTime = args.t[-1]
xs,_,ys = GetPosition(lastTime)
xs = (np.array(xs)-x0)/L
ys = (np.array(ys)-y0)/L
rs = GetRadius(lastTime)/L
qs = GetQuaternion(lastTime)
for xi,yi,ri in zip(xs,ys,rs):
	c = plt.Circle((xi,yi),ri,color='lightgray')
	ax.add_patch(c)

#Plot bead orientation at t = last time
for i,_ in enumerate(xs):
	ex = np.array([1,0,0])
	qi = qs[i]
	print(qi)
	Ri = qi.rotation_matrix
	dx,_,dy = Ri.dot(ex)
	x = np.array([xs[i],xs[i]+(rs[i])*dx])
	y = np.array([ys[i],ys[i]+(rs[i])*dy])
	ax.plot(x,y,'-',color='black',lw=0.5)

#Plot convergence |X_{t+dt} - X_{t}|
if (args.t.size)>5:
	epsilon = np.empty(args.t.size-1)
	for i,t in enumerate(args.t[1:]):
		dt = args.t[1]-args.t[0]
		xs_old,_,ys_old = GetPosition(t-dt)
		xs,_,ys = GetPosition(t)
		epsilon[i]= np.sqrt((xs-xs_old)**2 + (ys-ys_old)**2).sum()/L
	x_cvg = np.linspace(0,1,epsilon.size,endpoint=True)
	y_cvg = -epsilon/epsilon.max()
	ax.plot(x_cvg,y_cvg,'-',markersize=2,color='blue')

fig.savefig('plot.pdf')
