#!/usr/bin/python3
import numpy as np
import argparse
import sys
from numpy import linalg as LA
from scipy import interpolate
from scipy.linalg import lstsq
from pyquaternion import Quaternion
from Utils import *

def SmoothPolyline(polyline,s=1,info=True):
	if (info):
		print("  Smooth polyline")
	xs = polyline[:,0]
	ys = polyline[:,1]
	zs = polyline[:,2]
	periodic = 0
	if LA.norm(polyline[0]-polyline[-1]) < 1e-10:
		print("  periodic smoothing")
		periodic = 1
	tck,u = interpolate.splprep([xs,ys,zs],s=s,per=periodic,quiet=False)
	if periodic == 0:
		x,y,z,first_pt,last_pt = 0,1,2,0,-1
		tck[1][x][first_pt] = polyline[first_pt][x]
		tck[1][y][first_pt] = polyline[first_pt][y]
		tck[1][z][first_pt] = polyline[first_pt][z]
		tck[1][x][ last_pt] = polyline[ last_pt][x]
		tck[1][y][ last_pt] = polyline[ last_pt][y]
		tck[1][z][ last_pt] = polyline[ last_pt][z]
	xs,ys,zs = interpolate.splev(u,tck)
	return np.transpose([xs,ys,zs])

def VTKScript(polyline,out='tpcl'):
	original_stdout = sys.stdout
	with open(out + '.py', 'w') as f:
		sys.stdout = f
		vtkArray='[0,0,0]'
		if polyline.size > 0:
			vtkArray = np.array2string(polyline.ravel(),separator=',',max_line_width=np.inf,threshold=np.inf)
		vtkFile = out + '.vtk'
		print("from paraview.simple import *")
		print("polyline = PolyLineSource(registrationName='polyline')")
		print("polyline.Points =",vtkArray)
		print("SaveData('{out}', proxy=polyline)".format(out=vtkFile))
		print('print("  paraview --data={out}")'.format(out=vtkFile))
	sys.stdout = original_stdout
	return(0)

def VTKScripts(name='tpcl',cloudsize=-1,times=-1,smooth=-1,out='tpcl'):
	for time in times:
		polylines = ImportPolylines(my_path='.',name=name,cloudsize=cloudsize,time=time,smooth=smooth,info=False)
		for pl,polyline in enumerate(polylines):
			pExtension = ''
			tExtension = ''
			if cloudsize > 0:
				pExtension = '{p:02}'.format(p=pl)
			if time>-1:
				tExtension = '_t{t:05d}'.format(t=time)
			if out[-4:] == ".txt":
				out = out[:-4]
			pvout = out + pExtension + tExtension
			print("  pvpython",pvout+'.py')
			VTKScript(polyline,pvout)

def PrintCapillaryForce(name='tpcl',theta=90,ec=1,times=-1,info=True):
	print("#t Fx Fy Fz Tx Ty Tz")
	for time in times:
		F,T = GetNormalizedCapillaryForce(my_path=".",name='tpcl',theta=theta,ec=ec,time=time,info=info)
		print("{:d} {:.8f} {:.8f} {:.8f} {:.8f} {:.8f} {:.8f}".format(time,F[0],F[1],F[2],T[0],T[1],T[2]))

#Import contact line
parser = argparse.ArgumentParser()
parser.add_argument('-f',type=str,default='tpcl',help='cloud or single polyline')
parser.add_argument('-n',type=int,default=-1,help='max number of polylines in the cloud')
parser.add_argument('-t',type=StrToNumpy,default='-1',help='time')
parser.add_argument('-s',type=float,default=-1,help='smooth')
parser.add_argument('-ca',type=float,default=90,help='contact angle')
parser.add_argument('-ec',type=float,default=90,help='Ec')
parser.add_argument('-force',type=StrToBool,default='n',help='print force')
args = parser.parse_args()
cloud = args.f
cloudsize = args.n
times = args.t
smooth = args.s
theta = args.ca
ec = args.ec
out = args.f

if args.force:
	PrintCapillaryForce(name=cloud,theta=theta,ec=ec,times=times,info=False)
	#tend=126; ExportPolylines -c tpcl -t 0:${tend}; PolylineToVTK.py -c tpcl -t 0:${tend} -n 3; ExportSolid -s side0 -tp -t 0:${tend} -txt -stl;
	#PolylineToVTK.py -c 'tpcl' -n 3 -t 0:${tend} -f y > tmp
	#reset; plot 'tmp' u (20*$1):($2) ti 'py', 'force.txt' u 1:3 w l ti 'cpp'
	#reset; plot 'tmp' u (20*$1):($4) ti 'py', 'force.txt' u 1:5 w l ti 'cpp'
	#reset; plot 'tmp' u (20*$1):(sqrt($2**2+$4**2)) ti 'py','force.txt' u ($1):(sqrt($3**2+$5**2)) w l ti 'cpp'
	#reset; plot 'tmp' u (20*$1):($6) ti 'py', 'force.txt' u 1:7 w l ti 'cpp'

else:
	VTKScripts(name=cloud,cloudsize=cloudsize,times=times,smooth=smooth,out=out)
