from Utils import *
import numpy as np
import matplotlib
import matplotlib.pyplot as plt
import argparse
from pyquaternion import Quaternion
from scipy.optimize import curve_fit
matplotlib.rc('text', usetex=True)
matplotlib.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}"]
import os

#~~~~~~~~~~
#  Parser
#~~~~~~~~~~
parser = argparse.ArgumentParser()
parser.add_argument('-ca',type=int,default=90,help='contact angle')
parser.add_argument('-tmax',type=int,default=380,help='time')
args = parser.parse_args()

#~~~~~~
#  Ax
#~~~~~~
fig, ax = plt.subplots()
my_font_size = 14
my_frame_width = 0.5
xticks = np.linspace(0,4,9)
xticklabels = [r'${:.1f}$'.format(x) for x in xticks]
#ax.set_xscale('log')
yticks =  np.arange(0,111,25)
yticklabels = [r'${:d}^\circ$'.format(int(y)) for y in yticks]
ax.set_xlim(xticks[0],xticks[-1])
ax.set_ylim(yticks[0],yticks[-1])
ax.set_xticks(xticks)
ax.set_yticks(yticks)
ax.set_xticklabels(xticklabels)
ax.set_yticklabels(yticklabels)
ax.set_xlabel(r'$Ec$',fontsize=10,labelpad=4)
ax.set_ylabel(r'$\alpha_\infty$',fontsize=10,labelpad=2)
ax.xaxis.set_tick_params(which='major',length=4,direction='in',width=my_frame_width,pad=5)
ax.yaxis.set_tick_params(which='major',length=4,direction='in',width=my_frame_width,pad=5)
ax.xaxis.set_tick_params(which='minor',length=2,direction='in',width=my_frame_width,pad=5)
ax.yaxis.set_tick_params(which='minor',length=2,direction='in',width=my_frame_width,pad=5)
ax.xaxis.set_ticks_position('both')
ax.yaxis.set_ticks_position('both')
for axis in ['top','bottom','left','right']:
	ax.spines[axis].set_linewidth(my_frame_width)
for item in ([ax.title, ax.xaxis.label, ax.yaxis.label] + ax.get_xticklabels() + ax.get_yticklabels()):
	item.set_fontsize(my_font_size)

#~~~~~~~~~~~~~
#  Functions
#~~~~~~~~~~~~~
def GetFolderss(ca=90,es = np.arange(2,41,2)/10):
	folders_s3 = ["ca{}/side3/ec{:.1f}".format(ca,e) for e in es]
	folders_s4 = ["ca{}/side4/ec{:.1f}".format(ca,e) for e in es]
	folders_s5 = ["ca{}/side5/ec{:.1f}".format(ca,e) for e in es]
	folders_s6 = ["ca{}/side6/ec{:.1f}".format(ca,e) for e in es]
	folders_s7 = ["ca{}/side7/ec{:.1f}".format(ca,e) for e in es]
	return [folders_s3,folders_s4,folders_s5,folders_s6,folders_s7]

def GetAlphass(folderss,time):
	def GetAlpha(my_path,time):
		side_name = GetFileName(my_path + "/side0",time)
		if not os.path.isfile(side_name):
			print("  {} does not exist".format(side_name))
			return np.nan
		else:
			data = np.loadtxt(side_name)
			q = Quaternion(data[19],data[16],data[17],data[18])
			return  q.degrees
	alphass = np.empty((len(folderss),len(folderss[0])))
	for i,folders in enumerate(folderss):
		for j,folder in enumerate(folders):
			alphass[i][j] = GetAlpha(folder, time)
	return alphass

def CorrectData(es,alphass):
	es = np.concatenate(([0],es))
	alphass = np.hstack((np.zeros((len(alphass),1)),alphass))#.tolist
	ess = [es[~np.isnan(alphas)] for alphas in alphass]
	alphass = [alphas[~np.isnan(alphas)] for alphas in alphass]
	return ess,alphass

#~~~~~~~~
#  Data
#~~~~~~~~
last_time = args.tmax
es = np.arange(1,41,1)/10
folderss = GetFolderss(ca=args.ca,es=es)
alphass = GetAlphass(folderss,last_time)
ess,alphass = CorrectData(es,alphass)

#~~~~~~~~
#  Plot
#~~~~~~~~
colors = ['#c6dbef','#9ecae1','#6baed6','#3182bd','#08519c']
#colors = ['#ffffd4','#fee391','#fec44f','#fe9929','#ec7014']
sides = [3,4,5,6,7]
for i,(es,alphas) in enumerate(zip(ess,alphass)):
	ax.plot(es,
	        alphas,
	        marker = 'o',
	        markersize = 5,
	        markerfacecolor = 'white',
	        markeredgecolor = colors[i],
	        markeredgewidth = 1,
	        linewidth = 0,
	        label = r'$N = {}$'.format(sides[i]))

#~~~~~~~~~~~~
#  Best fit
#~~~~~~~~~~~~
def func(x,a,b):
	return a*(x)+b*(x**2)
alphass_extract = alphass.copy()
alphas_critical = np.empty((len(alphass)))
cut_off = alphass[0][-1]
for i,alphas in enumerate(alphass_extract):
	alphas_grad = np.array([np.abs(nexta - a)/nexta for a,nexta in zip(alphas[:-1],alphas[1:])])
	eps = 1e-2
	crop = np.argwhere(alphas_grad>eps).flatten()[-1]
	alphass_extract[i] = np.array(alphas[:crop+2])
for i,alphas in enumerate(alphass_extract):
	if alphas.size:
		popt, pcov = curve_fit(func, es[:len(alphas)],alphas)
		xs_fit = np.linspace(0,10,num=5000)
		ys_fit = func(xs_fit, *popt)
		index = np.argwhere(ys_fit<cut_off).flatten()[-1]
		alphas_critical[i] = xs_fit[index]
		ys_fit = np.where(ys_fit < cut_off, ys_fit, cut_off)
		ax.plot(xs_fit,ys_fit,'-',color=colors[i],zorder=0)

print('alpha_c = ',alphas_critical)

for i,(alphas,alphac) in enumerate(zip(alphass,alphas_critical)):
	A = [alphac+0.1,alphas[-1]-10]
	B = [alphac    ,alphas[-1]+0 ]
	Arrow(ax,A,B,arrowstyle='-|>',color=colors[i],head_width=0.2,head_length=0.4,shrinkA=3.,shrinkB=5.0,text=r'$\tilde{E}c$',zorder=10)

#~~~~~~~~~~
#  Legend
#~~~~~~~~~~
ax.legend()
handles, labels = ax.get_legend_handles_labels()
#sort=[3,2,1,0]
#labels = [labels[i] for i in sort]
#handles = [handles[i] for i in sort]
ax.legend(handles,
	      labels,
	      fontsize = my_font_size,
	      loc = 'lower right',
	      frameon = False,
	      handletextpad = 0.2,
	      handlelength = 2.5)

fig.savefig('plot.pdf')
plt.close(fig)
