#!/usr/bin/python3
import numpy as np
import matplotlib.pyplot as plt
import matplotlib
from scipy.optimize import curve_fit
matplotlib.rc('text', usetex=True)
matplotlib.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}"]
matplotlib.rcParams['text.latex.preamble']=[r"\usepackage{bm}"]
matplotlib.rc('text', usetex=True)


xs = [3,4,5,6,7]
ys70 = [0.93618724,1.60832166,2.24444889,2.87657532,3.46869374]
ys90 = [0.70414083,1.22224445,1.88037608,2.40448090,3.04060812]
colors=['#9ecae1','#08519c']
labels=[r'$\gamma = 70^\circ$',r'$\gamma = 90^\circ$']
markers=['^','o']

#~~~~~~
#  Ax
#~~~~~~
fig, ax = plt.subplots()
my_font_size = 14
my_frame_width = 0.5
xticks = xs
xticklabels = [r'${:d}$'.format(x) for x in xticks]
#ax.set_xscale('log')
yticks =  [0.5,1.0,1.5,2.0,2.5,3.0,3.5]
yticklabels = [r'${:.1f}$'.format(y) for y in yticks]
ax.set_xlim(xticks[0],xticks[-1])
ax.set_ylim(yticks[0],yticks[-1])
ax.set_xticks(xticks)
ax.set_yticks(yticks)
ax.set_xticklabels(xticklabels)
ax.set_yticklabels(yticklabels)
ax.set_xlabel(r'$N$' ,fontsize=my_font_size,labelpad=8)
ax.set_ylabel(r'$\tilde{E}c$',fontsize=my_font_size,labelpad=8)
ax.xaxis.set_tick_params(which='major',length=4,direction='in',width=my_frame_width,pad=8)
ax.yaxis.set_tick_params(which='major',length=4,direction='in',width=my_frame_width,pad=8)
ax.xaxis.set_tick_params(which='minor',length=2,direction='in',width=my_frame_width)
ax.yaxis.set_tick_params(which='minor',length=2,direction='in',width=my_frame_width)
ax.xaxis.set_ticks_position('both')
ax.yaxis.set_ticks_position('both')
for axis in ['top','bottom','left','right']:
	ax.spines[axis].set_linewidth(my_frame_width)
for item in ([ax.title, ax.xaxis.label, ax.yaxis.label] + ax.get_xticklabels() + ax.get_yticklabels()):
	item.set_fontsize(my_font_size)


def func(x,a,b):
	return a*(x)+b

xs_fit = np.linspace(xs[0],xs[-1],num=1000)
for i,ys in enumerate([ys70,ys90]):
	popt, pcov = curve_fit(func,xs,ys)
	ys_fit = func(xs_fit, *popt)
	ax.plot(xs_fit,ys_fit,'--',color=colors[i])
	ax.plot(xs,
	        ys,
	        marker = markers[i],
	        markersize = 10,
	        markerfacecolor = 'white',
	        markeredgecolor = colors[i],
	        markeredgewidth = 1,
	        linewidth = 0,
	        label = labels[i])

ax.legend()
handles, labels = ax.get_legend_handles_labels()
ax.legend(handles,
	      labels,
	      loc='upper left',
	      frameon = False,
	      handletextpad = 0.5,
	      handlelength = 0.5,
	      fontsize=my_font_size)


fig.savefig('plot.pdf')
plt.close(fig)

