import numpy as np
import matplotlib.pyplot as plt
from matplotlib.textpath import TextPath
from matplotlib.collections import PatchCollection
from matplotlib.patches import PathPatch, FancyBboxPatch
from matplotlib.patches import Polygon
from Utils import *

#INPUT DATA
Lx = 200
nmin = 3
nmax = 5
base_area = (Lx/10)**2 #base area
pyramide_height = Lx/3 #pyramid pyramide_height
gap = 2
separation = Lx
text_size = 15
pi = np.pi
images=['fig1_N3.png','fig1_N4.png','fig1_N5.png']

#Plot
fig, ax = plt.subplots(figsize=(20,20))
plt.axis('off')
ax.set_aspect('equal')
ax.set_xlim([-0.5*Lx,2.5*Lx])
ax.set_ylim([-0.5*Lx,0.5*Lx])


patches=[]
for i,n in enumerate(range(nmin,nmax+1)):
	# https://en.wikipedia.org/wiki/Regular_polygon
	# alpha   = (2pi)/n
	# alpha/2 = ( pi)/n
	# Base area A = n x (area of a single triangle) = n x [ a x (s/2) ]
	# Base area A = n (r^2) x sin(pi/n) cos(pi/n) = (n/2) (r^2) sin(2pi/n)
	# Base area A = (n/2) (r^2) x sin(2pi/n)
	# Base area A = (n/4) (s^2) / tan( pi/n)
	# Base area A = (n  ) (a^2) x tan( pi/n)
	# Radius      r =                                    = sqrt[ 2 (area/n) / sin(2pi/n) ]
	# Side length s = (2r) sin(alpha/2) = (2r) sin(pi/n) = sqrt[ 4 (area/n) x tan( pi/n) ]
	# Apothem     a = ( r) cos(alpha/2) = ( r) cos(pi/n) = sqrt[   (area/n) / tan( pi/n) ]
	base = np.empty((n+1,2))
	alphas = np.linspace(0,2*pi,num=n+1,endpoint=True)
	alpha1 = alphas[1]
	alphas = alphas - alpha1/2 + pi/2
	radius2 = 2*(base_area/n) / np.sin(alpha1)
	radius = np.sqrt(radius2)
	base[:,0] = radius * np.cos(alphas) + i*separation
	base[:,1] = radius * np.sin(alphas)
	centre = np.mean(base[:-1],axis=0)
	#ax.plot(base[:,0],base[:,1])
	r = radius
	s = (2*r)*np.sin(pi/n)
	a = (  r)*np.cos(pi/n)
	h = pyramide_height
	b = np.sqrt(h**2 + a**2)
	print(" n = {}".format(n))
	print(" A = {:4.3f} {:4.3f} {:4.3f} {:4.3f}".format(  n*a*s/2  ,  n/2*(r**2)*np.sin(2*pi/n) , n/4*(s**2)/np.tan(pi/n) , n*(a**2)*np.tan(pi/n)  ))
	print(" h = {:4.3f}".format(h))
	print(" r = {:4.3f} {:4.3f} {:4.3f}".format( r , np.sqrt(2*(base_area/n)/np.sin(2*pi/n)) , np.linalg.norm((        base[0])   - centre)  ))
	print(" s = {:4.3f} {:4.3f} {:4.3f}".format( s , np.sqrt(4*(base_area/n)*np.tan(  pi/n)) , np.linalg.norm((base[1]-base[0])           )  ))
	print(" a = {:4.3f} {:4.3f} {:4.3f}".format( a , np.sqrt(  (base_area/n)/np.tan(  pi/n)) , np.linalg.norm((base[1]+base[0])/2 - centre)  ))
	print(" b = {:4.3f}".format(b))
	polygon = Polygon(base)
	patches.append(polygon)
	label_position = centre-1.4*np.array([0,pyramide_height])
	TextBox(ax,label_position,r"$N={nside}$".format(nside=n),size=text_size,linewidth=0,pad=0.1)
	apothem2 = (base_area/n) / np.tan(pi/n)
	apothem = np.sqrt(apothem2)
	side_height = np.sqrt(pyramide_height**2 + apothem2)

	#Patches
	for side in range(n):
		triangle = np.empty((4,2))
		A = np.concatenate((base[side  ],[0]))
		B = np.concatenate((base[side+1],[0]))
		vn = np.cross(B-A,[0,0,1])
		vn = (vn/np.linalg.norm(vn))[:-1]
		vt = B-A
		vt = (vt/np.linalg.norm(vt))[:-1]
		A = A[:-1] + gap*vn
		B = B[:-1] + gap*vn
		C = (A+B)/2 + side_height*vn
		triangle[0] = A
		triangle[1] = B
		triangle[2] = C
		triangle[3] = triangle[0]
		polygon = Polygon(triangle)
		patches.append(polygon)
		l = np.linalg.norm(B-A)/4
		Arrow(ax,(A+B)/2-gap*vn+l*vt,(A+B)/2+l*vt,color='C0')
		Arrow(ax,(A+B)/2-gap*vn-l*vt,(A+B)/2-l*vt+gap*vn,color='C0')
		if side==0:
			#height of side triangle (b)
			A = triangle[0] - 2*gap*vt
			B = triangle[0] - 2*gap*vt + side_height*vn
			Arrow(ax,A,B,arrowstyle='<|-|>',head_width=0.3,head_length=0.6,linewidth=1)
			TextBox(ax,(A+B)/2,r"$b$",size=text_size,linewidth=0,pad=0.05)
		if side==0:
			#length of base side (s)
			A = triangle[0] + (side_height+2*gap)*vn
			B = triangle[1] + (side_height+2*gap)*vn
			Arrow(ax,A,B,arrowstyle='<|-|>',head_width=0.3,head_length=0.6,linewidth=1)
			TextBox(ax,(A+B)/2,r"$s$",size=text_size,linewidth=0,pad=0.05,zorder=3)
		if side==1:
			#apothem
			A0 = centre + 0.0*radius*vt
			B0 = centre + 0.0*radius*vt + apothem*vn
			Arrow(ax,A0,B0,arrowstyle='-',head_width=0.3,head_length=0.6,linewidth=1,zorder=2)
			A = A0 - 0.5*apothem*vn
			B = A0 + 0.05*apothem*vn
			Arrow(ax,A,B,arrowstyle='-|>',head_width=0.3,head_length=0.6,linewidth=1,zorder=2)
			A = B0 + 0.5*apothem*vn
			B = B0 - 0.05*apothem*vn
			Arrow(ax,A,B,arrowstyle='-|>',head_width=0.3,head_length=0.6,linewidth=1,zorder=2)
			A = (A0+B0)/2 + 0.0*radius*vt
			TextBox(ax,A,r"$a$",size=text_size,linewidth=0,pad=0.02,facecolor='C0',zorder=2)

	#Overlay snapshot
	centre_transformed = ax.transLimits.transform(centre)
	img_size = 0.2
	bbox = ax.get_position()
	x0 = bbox.x0 + bbox.width*centre_transformed[0] - img_size/2
	y0 = bbox.y1 - 0.05
	height = img_size
	width = img_size
	new_ax = fig.add_axes([x0,y0,height,width])
	new_ax.axis('off')
	img = plt.imread(images[i]);
	new_ax.imshow(img)

p = PatchCollection(patches,fc='C0',zorder=1)
ax.add_collection(p)
fig.savefig('fig1.pdf',dpi=400)
