from Utils import *
import numpy as np
import matplotlib
import matplotlib.pyplot as plt
import argparse
from pyquaternion import Quaternion
from scipy.optimize import curve_fit

def GetSide(fname,time):
	if time<0:
		fname = fname + '.txt'
	else:
		fname = fname + '_t{t:05d}.txt'.format(t=time)
	data = np.loadtxt(fname,usecols=0,unpack=True)
	m = np.reshape(data[0:9],(3,3)).transpose()
	h = data[9]
	n = data[10:13]
	c = data[13:16]
	q = Quaternion(data[19],data[16],data[17],data[18])
	m = np.array([c + q.rotate(row) for row in m])
	n = q.rotate(n)
	return m,h,n,c,q

def GetFiber(fname,time):
	fname = fname + '_t{time:05}.txt'.format(time=time)
	r,x,y,z,qx,qy,qz,qw = np.loadtxt(fname,usecols=(1,4,5,6,7,8,9,10),unpack=True)
	length = (r.size-1) * (2*r[0])
	rs = r / length
	pts = np.stack([x,y,z],axis=-1)
	pts = (pts - pts[0])/ length
	qs = np.stack((qw,qx,qy,qz),axis=-1)
	qs = [Quaternion(q) for q in qs]
	return rs[0], pts, qs

parser = argparse.ArgumentParser()
parser.add_argument('-t',type=StrToNumpy,default='0:600:10',help='time')
args = parser.parse_args()

es = np.array([0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1.0,1.1,1.2,1.4,1.6,1.8,2.0])
folders_3 = ["side3/Ec{:.1f}".format(e) for e in es]
folders_4 = ["side4/Ec{:.1f}".format(e) for e in es]
folders_5 = ["side5/Ec{:.1f}".format(e) for e in es]

#Arrays
times = args.t
qs = np.empty((times.size))
qs_inf = np.empty(es.size)
fig, axs = plt.subplots(3,2)

#Min/max height
#m,_,_,_,_ = GetSide(folders_3[-1]+"/side0",0)
#hmin = m[-1][-1]
#m,_,_,_,q = GetSide(folders_3[-1]+"/side0",599)
#hmax = m[-1][-1]

cuta = 90 #1.1
cutb = 100

#Plots
for row,folders in enumerate([folders_3,folders_4,folders_5]):
	for fi,folder in enumerate(folders):
		for ti,time in enumerate(times):
			m,h,n,c,q = GetSide(folder + "/side0", time)
			qs[ti] = q.degrees
		axs[row,0].plot(times,qs,'-') #color=colors[fi])
		qs_inf[fi] = qs[-1]
	crop = (np.abs(qs_inf-qs_inf[-1]) > 1).nonzero()[0][-1]
	crop = crop+1
	es_extract = np.concatenate(([0],es    [:crop+1]))
	qs_extract = np.concatenate(([0],qs_inf[:crop+1]))
	#print("qs_up =",qs_extract)
	def func(x,a,b):
		return a*x+b*x**2
	popt, pcov = curve_fit(func, es_extract, qs_extract)
	x_fit = np.linspace(0,2)
	y_fit = func(x_fit, *popt)
	x_fit = x_fit[y_fit<cutb] #1.1]
	y_fit = y_fit[y_fit<cutb] #1.1]
	axs[row,1].plot(x_fit,y_fit,'-')
	axs[row,1].plot(np.concatenate(([0],es)),np.concatenate(([0],qs_inf)),'+')

fig.savefig('plot.pdf')
plt.close(fig)

